<?php
namespace Teknisa\CMP\Modules\Solicitacao\Modules\SolicitacaoCompras\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Exception;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\DateUtil;
use \Setforms2\Shared\ORM\DateTime;
use Teknisa\CMP\Modules\Solicitacao\Modules\SolicitacaoCompras\Services\CMP06900AlteracaoAutomaticaSolicitacaoComprasService;

class CMP06900AlteracaoAutomaticaSolicitacaoComprasController {

    protected $birtStrategy;

    public function __construct(CMP06900AlteracaoAutomaticaSolicitacaoComprasService $service, $dataprocessamento) {
        $this->service = $service;
        $this->dataprocessamento = $dataprocessamento;
    }
    
    private $mensagensPadrao = array(
        'Entrega_ED'               => 'Alteração da Entrega OL para Entrega ED',
        'Entrega_OL'               => 'Alteração da Entrega ED para Entrega OL',
        'Alt_Auto'                 => 'Alteração Automática',
        'N_Sol_selec'              => 'Não há solicitação selecionada.',
        'Just_DINF'                => 'Justificativa deve ser informada.',
        'Quant_Cons'               => 'Quantidade já considerada na consolidação do OL nº %s.',
        'N_Altera_Qtde'            => 'Operação bloqueada. Parâmetro "Permitir alteração de quantidade" está desmarcado.',
        'S_Permiss_Alter_Qtde'     => 'Operação bloqueada. Quantidade não pode ser alterada para %s.',
        'Just_Padrao_AlterGrid'    => 'Alteração - CMP06900',
        'FILI_N_OL'                => 'Marca não pode ser incluida. Filial solicitante não é uma filial OL.',
        'Just_Padrao_DtEntrega'    => 'Dt. Entrega de: %s p/ %s',
        'PesqJustSoli'             => 'Justificativa de Alteração',
        'Confirmar_DataEntrega'    => 'Confirmar alteração da data de entrega dos itens selecionados.',
        'Confirmar_MultDataEntrega'=> 'Houve seleção de entregas distintas para alteração automática. Deseja confirmar a operação?'

    );
    
    public function getCodigoGrupProd(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $result = $this->service->getCodigoGrupProd($row);
        } catch (Exception $e) {
            $response->setError(Error::factoryFromException($e, $e->getMessage(), $e->getCode()));
        }
        $response->addDataSet(new DataSet('data', [$result]));
    }
    
    /**
     * Agrupar as funções de acordo com os fluxos:
     * - Gerais (independe de fluxo);
     * - Atualização do Grid;
     * - Alteração Automática de Data de Entrega;
     * - Alteração Automática de Produto;
     * - Utilidades;
     */
     /**
     * ########################################################################
     * ################## >>> ALTERAR ENTREGA ED/OL <<< #######################
     * ########################################################################
     */
     
    /**
     * Necessário pois a função 'salvarAlteracoesSolicitacao' os utiliza e,
     * no caso do fluxo 'Atualização do Grid', é tratado no front-end, porque 
     * somente um item é passado. E neste fluxo de 'Alterar Entrega ED/OL', 
     * pode haver um ou mais itens selecionados.
     */
    private function setOldParams(&$item) {
        $item["STATUS_OLD"]      = $item["STATUS"];
        $item["CDFILSOLI_OLD"]   = $item["CDFILSOLI"];
        $item["CDFILLAN_OLD"]    = $item["CDFILLAN"];
        $item["NRSOLICMP_OLD"]   = $item["NRSOLICMP"];
        $item["CDPRODUTO_OLD"]   = $item["CDPRODUTO"];
        $item["DTPROGENTR_OLD"]  = $item["DTPROGENTR"];
        $item["QTSOLIPROD_OLD"]  = $item["QTSOLIPROD"];
    }
    
    private function setOperador(&$item, $rowCDOPERADOR) {
        $item["CDOPERADOR"] = $rowCDOPERADOR;
    }
     
    public function podeAlterarEntrega($row) {
        if(is_array($row) && !is_null($row)) {
            if (count($row["ITENS_SELECIONADOS"]) <= 0) {
                throw new Exception($this->getMensagemPadrao('N_Sol_selec'));
            }
        } else {
            throw new Exception($this->getMensagemPadrao('N_Sol_selec'));
        }
    }
    
    public function alterarEntrega($row) {
        try {
            foreach ($row["ITENS_SELECIONADOS"] as $item) {
                self::setOperador($item, $row["CDOPERADOR"]);
                self::setOldParams($item);
                
                $item["STATUS"] = $row["NEW_STATUS"];
                
                $this->salvarAlteracoesSolicitacao($item);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function acionaAlterarEntrega(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $this->service->beginTransaction();
            $this->podeAlterarEntrega($row);
            $this->alterarEntrega($row);
            $this->service->realizaCommit();
        } catch (Exception $e) {
            $this->service->realizaRollback();
            $response->setError(Error::factoryFromException($e, $e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * ########################################################################
     * ############### >>> ATUALIZAÇÃO DE ITENS DO GRID <<< ###################
     * ########################################################################
     * 
     * Responsável por atualizar:
     * - Quantidade;
     * - Marca;
     * - Justificativa.
     */
     
	private function validaMarca($row) {
	    if ($row['CDMARCA'] != $row['CDMARCA_OLD']) {
            $params = array(
                "P_CDFILIAL" => $row['CDFILSOLI']
            );
            $tipoFilial = $this->service->qryTipoFilial($params);
            
            if (($tipoFilial['IDFUNCAOFILI'] != 'O') && !empty($row['CDMARCA'])) {
                throw new Exception(self::getMensagemPadrao('FILI_N_OL'));
            }
	    }
	}
	
	private function validaJustificativa($row) {
	    if ($row["IDAUDALTSOL"] == 'S') {
	        if ($row["ALTERADO"] == 'S') {
	            if (empty($row["DSJUSTOPER"])) {
	                throw new Error(self::getMensagemPadrao('Just_DINF'));
	            }
	        }
	    }
	}
	
    public function validaRegraAlterarSolicitacaoGrid($item) {
        $regras = array();
        
        $autGrSoli = $this->service->adqAutGrSoliIDAUTALTESOLI();
        
        if (empty($autGrSoli)) {
            $regras["boAltDataEntrega"] = true;
            $regras["boAumentQtde"]     = true;
            $regras["boDiminuQtde"]     = true;
        } else {
            $IDAUTALTESOLI_ALTERAR_DTENTREGA    =  false;
            $IDAUTALTESOLI_AUMENTAR_QUANTIDADE  =  false;
            $IDAUTALTESOLI_DIMINUIR_QUANTIDADE  =  false;
            
            foreach ($autGrSoli as $regra) {
                switch($regra["IDAUTALTESOLI"]) {
                    case "ALTERAR_DTENTREGA":
                        $IDAUTALTESOLI_ALTERAR_DTENTREGA = true;
                        break;
                    case "AUMENTAR_QUANTIDADE":
                        $IDAUTALTESOLI_AUMENTAR_QUANTIDADE = true;
                        break;
                    case "DIMINUIR_QUANTIDADE":
                        $IDAUTALTESOLI_DIMINUIR_QUANTIDADE = true;
                        break;
                }
            }
            
            $regras["boAltDataEntrega"] = (($item["IDTIPLAN"] != 'A') || ($item["IDTIPLAN"] == 'A' && $IDAUTALTESOLI_ALTERAR_DTENTREGA));
            $regras["boAumentQtde"]     = (($item["IDTIPLAN"] != 'A') || ($item["IDTIPLAN"] == 'A' && $IDAUTALTESOLI_AUMENTAR_QUANTIDADE));
            $regras["boDiminuQtde"]     = (($item["IDTIPLAN"] != 'A') || ($item["IDTIPLAN"] == 'A' && $IDAUTALTESOLI_DIMINUIR_QUANTIDADE));
        }
        
        return $regras;
    }
    
    public function validaRegraAlterarQuantidade($row, $response) {
        self::parseQuantidadesToFloat($row);
        $newQtSoliProd = $row['QTSOLIPROD'];
        $oldQtSoliProd = $row['QTSOLIPROD_OLD'];
        
        if ($newQtSoliProd != $oldQtSoliProd) {
            $dbNewValue = $row['QTSOLIPROD'];
            
            $regras = self::validaRegraAlterarSolicitacaoGrid($row);
        
            $autGrSoli = $this->service->adqAutGrSoliIDAUTALTESOLI();
            
            if (!$regras['boAumentQtde'] && $newQtSoliProd > $oldQtSoliProd && $newQtSoliProd != 0) {
                throw new Exception(sprintf(self::getMensagemPadrao('S_Permiss_Alter_Qtde'), $dbNewValue));
            }
            
            if (!$regras['boDiminuQtde'] && $newQtSoliProd < $oldQtSoliProd && $newQtSoliProd != 0) {
                throw new Exception(sprintf(self::getMensagemPadrao('S_Permiss_Alter_Qtde'), $dbNewValue));
            }
            
            if ($row['QTSOLIORIG'] > 0) {
                $dbQtOrig = $row['QTSOLIORIG'];
            } else {
                $dbQtOrig = $row['QTSOLIPROD_OLD'];
            }
            
            if ($regras['boAumentQtde']) {
                $itemAutGrSoli = self::searchInArray($autGrSoli, 'IDAUTALTESOLI', 'AUMENTAR_QUANTIDADE');
                if (!is_null($itemAutGrSoli)) {
                    if (!is_null($itemAutGrSoli['PEALTEQTDESOLI'])) {
                        if ($newQtSoliProd > ($dbQtOrig + ($dbQtOrig * ($itemAutGrSoli['PEALTEQTDESOLI'] / 100)))) {
                            throw new Exception(sprintf(self::getMensagemPadrao('S_Permiss_Alter_Qtde'), $dbNewValue));
                        }
                    }
                }
            }
            
            if ($regras['boDiminuQtde']) {
                $itemAutGrSoli = self::searchInArray($autGrSoli, 'IDAUTALTESOLI', 'DIMINUIR_QUANTIDADE');
                if (!is_null($itemAutGrSoli)) {
                    if (!is_null($itemAutGrSoli['PEALTEQTDESOLI'])) {
                        if ($newQtSoliProd < ($dbQtOrig - ($dbQtOrig * ($itemAutGrSoli['PEALTEQTDESOLI'] / 100))) && $newQtSoliProd != 0) {
                            throw new Exception(sprintf(self::getMensagemPadrao('S_Permiss_Alter_Qtde'), $dbNewValue));
                        }
                    }
                }
            }
            
            $stIdUtAprovSol = $row['IDUTAPROVSOL'];
            $stIdPerAltSApr = $row['IDPERALTSAPR'];
            $stIdSoliAprov  = $row['IDSOLIAPROV'];
            if ($stIdUtAprovSol  == 'S' && $stIdPerAltSApr[1] == 'N' && $stIdSoliAprov == 'S') {
                throw new Exception(self::getMensagemPadrao('N_Altera_Qtde'));
            }
            
            $row['ALTERADO'] = 'S';
            
            if ($row['IDAUDALTSOL'] == 'N') {
                $row['DSJUSTOPER'] = self::getMensagemPadrao('Just_Padrao_AlterGrid');
            }
            
            
            if (!empty($row['NRSEQUCONSOL'])) {
                throw new Exception(sprintf(self::getMensagemPadrao('Quant_Cons'), $row["NRSEQUCONSOL"]));
            }
            self::validaMarca($row);
            
            if (!empty($row['DSJUSTOPER'])) {
                self::validaJustificativa($row);
            }
        }
	}
    
    /**
     * Utilizada pelos seguintes procedimentos:
     * * Atualização de itens no Grid;
     * * Alterar Entrega ED/OL.
     */
    public function salvarAlteracoesSolicitacao($row) {
        $stNvStatusItmPdOL = null;
        $stStatusItmPdOL = null;
        
        self::parseDates($row);
        self::parseQuantidadesToFloat($row);
        
        try {
            if($row["QTSOLIPROD"] == 0) {
                $paramsAdqItemOrInsItemSoliDel = array(
                    "P_CDFILSOLI"       => $row["CDFILSOLI"],
                    "P_CDFILLAN"        => $row["CDFILLAN"],
                    "P_NRSOLICMP"       => $row["NRSOLICMP"],
                    "P_CDPRODUTO"       => $row["CDPRODUTO"],
                    "P_DTPROGENTR"      => $row["DTPROGENTR"]
                );
                
                $itemSoliDel = $this->service->adqItemsoliDel($paramsAdqItemOrInsItemSoliDel);
                
                if($itemSoliDel == []) {
                    if(isset($row['QTSOLIORIG'])) {
                        if($row['QTSOLIORIG'] == 0 || $row['QTSOLIORIG'] == null) {
                            $paramsAdqItemOrInsItemSoliDel['P_QTSOLIORIGDEL'] = $row['QTSOLIPROD'];
                        } else {
                            $paramsAdqItemOrInsItemSoliDel['P_QTSOLIORIGDEL'] = $row['QTSOLIORIG'];
                        }
                    } else {
                        $paramsAdqItemOrInsItemSoliDel['P_QTSOLIORIGDEL'] = $row['QTSOLIPROD'];
                    }
                    $this->service->acmInsItemSoliDel($paramsAdqItemOrInsItemSoliDel);
                }
                /*Cláusula else atrelada ao if ($itemSoliDel == []) foi removida
                  por ser uma funcionalidade que raramente seria utilizada, de
                  acordo com o Davidson no dia 26/10/2018.
                  Ela chamava o $this->service->acmUpdItemSoliDel($params).
                  A query não foi passada para a implementação.*/
                
                $paramsInsMovOpSol = array(
                    "P_CDFILSOLMOV"     => $row["CDFILSOLI"],
                    "P_CDFILLANMOV"     => $row["CDFILLAN"],
                    "P_NRSOLIMOV"       => $row["NRSOLICMP"],
                    "P_CDPRODUTO"       => $row["CDPRODUTO"],
                    "P_DTENTROPER"      => $row["DTPROGENTR"],
                    "P_CDOPERADOR"      => $row["CDOPERADOR"],
                    "P_QTANTOPER"       => $row["QTSOLIPROD"],
                    "P_QTDEPOPER"       => 0,
                    "P_CDFILIALOLANT"   => "",
                    "P_CDFILIALOLDEP"   => ""
                );
                
                if ($row['STATUS'] != $row['STATUS_OLD']) {
                   if ($row['STATUS'] == 'P') {
                      $paramsInsMovOpSol['P_DSJUSTOPER'] = $this->getMensagemPadrao('Entrega_ED');
                   } else if ($row['STATUS'] == 'O') {
                      $paramsInsMovOpSol['P_DSJUSTOPER'] = $this->getMensagemPadrao('Entrega_OL');
                   }
                } else {
                   $paramsInsMovOpSol['P_DSJUSTOPER'] = $row['DSJUSTOPER'];
                }
                
                if ($row['CDMARCA'] == null) {
                   $paramsInsMovOpSol['P_CDMARCA'] = null;
                } else {
                   $paramsInsMovOpSol['P_CDMARCA'] = $row['CDMARCA'];
                }
                
                $this->service->acmInsMovOpSol($paramsInsMovOpSol);
                
                $paramsDelItemSoli = array(
                    "P_CDFILSOLI"     => $row["CDFILSOLI_OLD"],
                    "P_CDFILLAN"      => $row["CDFILLAN_OLD"],
                    "P_NRSOLICMP"     => $row["NRSOLICMP_OLD"],
                    "P_CDPRODUTO"     => $row["CDPRODUTO_OLD"],
                    "P_DTPROGENTR"    => $row["DTPROGENTR_OLD"],
                );
                
                $this->service->acmDeleteItemSoli($paramsDelItemSoli);
            } else {
                $paramsUpdateItemSoli = array(
                    "CDFILSOLI"     => $row["CDFILSOLI_OLD"],
                    "CDFILLAN"      => $row["CDFILLAN_OLD"],
                    "NRSOLICMP"     => $row["NRSOLICMP_OLD"],
                    "CDPRODUTO"     => $row["CDPRODUTO_OLD"],
                    "DTPROGENTR"    => $row["DTPROGENTR_OLD"],
                    "QTSOLIPROD"    => $row["QTSOLIPROD"],
                );
                
                if ($row['QTSOLIPROD'] != $row['QTSOLIPROD_OLD']) {
                   $paramsUpdateItemSoli['QTSOLIORIG'] = $row['QTSOLIPROD_OLD'];
                } else {
                   $paramsUpdateItemSoli['QTSOLIORIG'] = null;
                }
                
                if ($row['CDMARCA'] == null) {
                   $paramsUpdateItemSoli['CDMARCA'] = null;
                } else {
                   $paramsUpdateItemSoli['CDMARCA'] = $row['CDMARCA'];
                }
                
                if ($row['STATUS'] == 'O') {
                    $paramsAdqFiliOpelog = array(
                        "P_CDFILIAL"    => $row["CDFILSOLI_OLD"],
                        "P_CDPRODUTO"   => $row["CDPRODUTO_OLD"]
                    );
                    
                    $cdFiliOpelog = $this->service->adqFiliOpelogCDFILIOPELOG($paramsAdqFiliOpelog);
                    $paramsUpdateItemSoli['CDFILIOPELOG'] = $cdFiliOpelog;
                    $stNvStatusItmPdOL  = $cdFiliOpelog;
                    $stStatusItmPdOL    = '';
                } else {
                    $paramsAdqSoliItemPedidoOL = array(
                        "CDFILSOLI" => $row["CDFILSOLI_OLD"],
                        "CDFILLAN"  => $row["CDFILLAN_OLD"],
                        "NRSOLICMP" => $row["NRSOLICMP_OLD"],
                        "CDPRODUTO" => $row["CDPRODUTO_OLD"],
                        "DTPROGENTR"    => $row["DTPROGENTR_OLD"],
                    );
                    
                    $paramsAdqFiliOpelog = array(
                        "P_CDFILIAL"    => $row["CDFILSOLI_OLD"],
                        "P_CDPRODUTO"   => $row["CDPRODUTO_OLD"]
                    );
                    
                    $pedidoOL = $this->service->adqSoliItemPedidoOL($paramsAdqSoliItemPedidoOL);
                    $cdFiliOpelog = $this->service->adqFiliOpelogCDFILIOPELOG($paramsAdqFiliOpelog);
                    
                    if ($pedidoOL['IDPEDIDOOL'] == 'N') {
                        $paramsUpdateItemSoli['CDFILIOPELOG'] = null;
                        $stNvStatusItmPdOL  = "";
                        $stStatusItmPdOL    = $cdFiliOpelog;
                    } else {
                        $paramsUpdateItemSoli['CDFILIOPELOG'] = $cdFiliOpelog;
                        $stNvStatusItmPdOL  = $cdFiliOpelog;
                        $stStatusItmPdOL    = "";
                    }
                }
                
                $this->service->acmUpdateItemSoli($paramsUpdateItemSoli);
                
                $paramsInsMovOpSol = array(
                    "P_CDFILSOLMOV"     => $row["CDFILSOLI_OLD"],
                    "P_CDFILLANMOV"     => $row["CDFILLAN_OLD"],
                    "P_NRSOLIMOV"       => $row["NRSOLICMP_OLD"],
                    "P_CDPRODUTO"       => $row["CDPRODUTO_OLD"],
                    "P_DTENTROPER"      => $row["DTPROGENTR_OLD"],
                    "P_CDOPERADOR"      => $row["CDOPERADOR"],
                    "P_QTANTOPER"       => $row["QTSOLIPROD_OLD"],
                    "P_QTDEPOPER"       => $row["QTSOLIPROD"]
                );
                
                if ($row['QTSOLIPROD'] != $row['QTSOLIPROD_OLD']) {
                    $stStatusItmPdOL = $stNvStatusItmPdOL;
                }
                
                if ($row['STATUS'] != $row['STATUS_OLD']) {
                   if ($row['STATUS'] == 'P') {
                      $paramsInsMovOpSol['P_DSJUSTOPER'] = $this->getMensagemPadrao('Entrega_ED');
                   } else if ($row['STATUS'] == 'O') {
                      $paramsInsMovOpSol['P_DSJUSTOPER'] = $this->getMensagemPadrao('Entrega_OL');
                   }
                } else {
                   $paramsInsMovOpSol['P_DSJUSTOPER'] = $row['DSJUSTOPER'];
                }
                
                if ($row['CDMARCA'] == null) {
                   $paramsInsMovOpSol['P_CDMARCA'] = null;
                } else {
                   $paramsInsMovOpSol['P_CDMARCA'] = $row['CDMARCA'];
                }
                
                $paramsInsMovOpSol['P_CDFILIALOLANT'] = $stStatusItmPdOL;
                $paramsInsMovOpSol['P_CDFILIALOLDEP'] = $stNvStatusItmPdOL;
                
                $this->service->acmInsMovOpSol($paramsInsMovOpSol);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function acionaSalvarAlteracoesSolicitacao(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        try {
            $this->service->beginTransaction();
            foreach ($row as $solicitacao){
                
                $this->validaRegraAlterarQuantidade($solicitacao, $response);
                $this->salvarAlteracoesSolicitacao($solicitacao);
            }
            $this->service->realizaCommit();
        } catch (Exception $e) {
            $this->service->realizaRollback();
            $response->setError(Error::factoryFromException($e, $e->getMessage(), $e->getCode()));
        } 
    }
     
    /**
     * ########################################################################
     * ########## >>> ATUALIZAÇÃO AUTOMÁTICA DE DATA DE ENTREGA <<< ###########
     * ########################################################################
     */
     
    private function verificaDataEntrega($row)
    {
        foreach ($row["ITENS_SELECIONADOS"] as $item) {
            $result = $this->dataprocessamento->getDataprocessamento($item["CDFILSOLI"]);
            $novaData = DateTime::createFromFormat("d/m/Y", $row["DTPROGENTR_NEW"]);
            if($novaData->format('Y/m/d') < $result) {
                throw new Exception('DATAPROC_MAIOR');       
            }
        }  
    }
     
    public function alteracaoAutomaticaDataEntrega($row) {
        try {
            foreach ($row["ITENS_SELECIONADOS"] as $item) {
                
                $this->verificaDataEntrega($row);
            
                self::parseDates($item);
                self::parseQuantidadesToFloat($item);
                
                $item["DSJUSTOPER"] = isset($row["DSJUSTOPER_NEW"]) ? $row["DSJUSTOPER_NEW"] : sprintf(self::getMensagemPadrao('Just_Padrao_DtEntrega'), $item["DTPROGENTR"], $row["DTPROGENTR_NEW"]);
                $item["STATUS_OLD"] = $item["STATUS"];
                
                $paramsVerifItemSoli = array(
                    "P_CDFILSOLI"   => $item["CDFILSOLI"],
                    "P_CDFILLAN"    => $item["CDFILLAN"],
                    "P_NRSOLICMP"   => $item["NRSOLICMP"],
                    "P_CDPRODUTO"   => $item["CDPRODUTO"],
                    "P_DTPROGENTR"  => $item["DTPROGENTR"]
                );
                
                $itemSoli = $this->service->adqVerifItemSoli($paramsVerifItemSoli);
                
                if (!empty($itemSoli)) {
                    if ($item["DTPROGENTR"] != $row["DTPROGENTR_NEW"]) {
                        /**
                         * Comentário do Delphi:
                         * 'verifica se existe o item da solicitação na data informada.'
                         */
                        $paramsAdqItemSoli = array(
                            "P_CDFILSOLI"   => $item["CDFILSOLI"],
                            "P_CDFILLAN"    => $item["CDFILLAN"],
                            "P_NRSOLICMP"   => $item["NRSOLICMP"],
                            "P_CDPRODUTO"   => $item["CDPRODUTO"],
                            "P_DTPROGENTR"  => $row["DTPROGENTR_NEW"]
                        );
                        
                        $itemSoli = $this->service->adqItemsoli($paramsAdqItemSoli);
                        
                        if (empty($itemSoli)) {
                            $paramsInsItemSoli = array(
                                "CDFILSOLI" => $item["CDFILSOLI"],
                                "CDFILLAN"  => $item["CDFILLAN"],
                                "NRSOLICMP" => $item["NRSOLICMP"],
                                "CDPRODUTO" => $item["CDPRODUTO"],
                                "DTPROGENTR"    => $row["DTPROGENTR_NEW"],
                                "DTUTILIZA" => $item["DTUTILIZA"],
                                "QTSOLIPROD"    => $item["QTSOLIPROD"],
                                "TXOBSITMSOL"   => $item["TXOBSITMSOL"],
                                "IDSOLIAPROV"   => $item["IDSOLIAPROV"],
                                "QTVOLUITEMSO"  => $item["QTVOLUITEMSO"],
                                "CDPRODORIG"    => $item["CDPRODORIG"],
                                "QTSOLIORIG"    => $item["QTSOLIORIG"],
                                "VRCMPANTERIOR" => $item["VRCMPANTERIOR"],
                                "DTCMPANTERIOR" => $item["DTCMPANTERIOR"],
                                "CDFILIOPELOG"  => $item["CDFILIOPELOG"],
                                "CDRASTRITM"    => $item["CDRASTRITM"]
                            );
                            /**
                             * Comentário do Delphi:
                             * 'insere o produto na soliticação na data informada'
                             */
                            $this->service->acmInsItemSoli($paramsInsItemSoli);
                            
                            $paramsInsMovOpSol = array(
                                "P_CDFILSOLMOV" => $item["CDFILSOLI"],
                                "P_CDFILLANMOV" => $item["CDFILLAN"],
                                "P_NRSOLIMOV"   => $item["NRSOLICMP"],
                                "P_CDPRODUTO"   => $item["CDPRODUTO"],
                                "P_DTENTROPER"  => $row["DTPROGENTR_NEW"],
                                "P_CDOPERADOR"  => $row["CDOPERADOR"],
                                "P_QTANTOPER"   => $item["QTSOLIPROD"],
                                "P_QTDEPOPER"   => $item["QTSOLIPROD"],
                                "P_CDFILIALOLANT"   => "",
                                "P_CDFILIALOLDEP"   => ""
                            );
                            
                            if ($item['STATUS'] != $item['STATUS_OLD']) {
                                if ($item['STATUS'] == 'P') {
                                    $paramsInsMovOpSol['P_DSJUSTOPER'] = $this->getMensagemPadrao('Entrega_ED');
                                } else if ($item['STATUS'] == 'O') {
                                    $paramsInsMovOpSol['P_DSJUSTOPER'] = $this->getMensagemPadrao('Entrega_OL');
                                }
                            } else {
                                $paramsInsMovOpSol['P_DSJUSTOPER'] = $item['DSJUSTOPER'];
                            }
                            
                            if (is_null($item['CDMARCA'])) {
                                $paramsInsMovOpSol['P_CDMARCA'] = "";
                            } else {
                                $paramsInsMovOpSol['P_CDMARCA'] = $item['CDMARCA'];
                            }
                            /**
                             * Comentário do Delphi:
                             * 'Insere log de alteração'
                             */
                            $this->service->acmInsMovOpSol($paramsInsMovOpSol);
                        } else {
                            $paramsUpdItemSoli = array(
                                "P_CDFILSOLI"   => $item["CDFILSOLI"],
                                "P_CDFILLAN"    => $item["CDFILLAN"],
                                "P_NRSOLICMP"   => $item["NRSOLICMP"],
                                "P_CDPRODUTO"   => $item["CDPRODUTO"],
                                "P_DTPROGENTR"  => $row["DTPROGENTR_NEW"],
                                "P_QTSOLIPROD"  => $item["QTSOLIPROD"]
                            );
                            /**
                             * Comentário do Delphi:
                             * 'altera a quantidade quando já existe o produto na solicitação na data informada'
                             */
                            $this->service->acmUpdItemSoli($paramsUpdItemSoli);
                            
                            /**
                             * Comentário do Delphi:
                             * 'Insere log de alteração'
                             */
                            $paramsInsMovOpSol = array(
                                "P_CDFILSOLMOV"     => $item["CDFILSOLI"],
                                "P_CDFILLANMOV"     => $item["CDFILLAN"],
                                "P_NRSOLIMOV"       => $item["NRSOLICMP"],
                                "P_CDPRODUTO"       => $item["CDPRODUTO"],
                                "P_DTENTROPER"      => $row["DTPROGENTR_NEW"],
                                "P_CDOPERADOR"      => $row["CDOPERADOR"],
                                "P_QTANTOPER"       => $itemSoli["QTSOLIPROD"],
                                "P_QTDEPOPER"       => $itemSoli["QTSOLIPROD"] + $item["QTSOLIPROD"],
                                "P_CDFILIALOLANT"   => "",
                                "P_CDFILIALOLDEP"   => ""
                            );
                            
                            if ($item['STATUS'] != $item['STATUS_OLD']) {
                                if ($item['STATUS'] == 'P') {
                                    $paramsInsMovOpSol['P_DSJUSTOPER'] = $this->getMensagemPadrao('Entrega_ED');
                                } else if ($item['STATUS'] == 'O') {
                                    $paramsInsMovOpSol['P_DSJUSTOPER'] = $this->getMensagemPadrao('Entrega_OL');
                                }
                            } else {
                                $paramsInsMovOpSol['P_DSJUSTOPER'] = $item['DSJUSTOPER'];
                            }
                            
                            if (is_null($item['CDMARCA'])) {
                                $paramsInsMovOpSol['P_CDMARCA'] = "";
                            } else {
                                $paramsInsMovOpSol['P_CDMARCA'] = $item['CDMARCA'];
                            }
                            
                            $this->service->acmInsMovOpSol($paramsInsMovOpSol);
                        }
                        
                        $paramsDeleteItemSoli = array(
                            "P_CDFILSOLI" => $item["CDFILSOLI"],
                            "P_CDFILLAN"  => $item["CDFILLAN"],
                            "P_NRSOLICMP" => $item["NRSOLICMP"],
                            "P_CDPRODUTO" => $item["CDPRODUTO"],
                            "P_DTPROGENTR"    => $item["DTPROGENTR"]
                        );
                        /**
                         * Comentário do Delphi:
                         * 'exclui o produto da data antiga'
                         */
                        $this->service->acmDeleteItemSoli($paramsDeleteItemSoli);
                        
                        /**
                         * Comentário do Delphi:
                         * 'Insere log de alteração'
                         */
                        $paramsInsMovOpSol = array(
                            "P_CDFILSOLMOV"   => $item["CDFILSOLI"],
                            "P_CDFILLANMOV"   => $item["CDFILLAN"],
                            "P_NRSOLIMOV"     => $item["NRSOLICMP"],
                            "P_CDPRODUTO"     => $item["CDPRODUTO"],
                            "P_DTENTROPER"    => $item["DTPROGENTR"],
                            "P_CDOPERADOR"    => $row["CDOPERADOR"],
                            "P_QTANTOPER"     => $item["QTSOLIPROD"],
                            "P_QTDEPOPER"     => 0,
                            "P_CDFILIALOLANT" => "",
                            "P_CDFILIALOLDEP" => "",
                        );
                        
                        if ($item['STATUS'] != $item['STATUS_OLD']) {
                            if ($item['STATUS'] == 'P') {
                                $paramsInsMovOpSol['P_DSJUSTOPER'] = $this->getMensagemPadrao('Entrega_ED');
                            } else if ($item['STATUS'] == 'O') {
                                $paramsInsMovOpSol['P_DSJUSTOPER'] = $this->getMensagemPadrao('Entrega_OL');
                            }
                        } else {
                            $paramsInsMovOpSol['P_DSJUSTOPER'] = $item['DSJUSTOPER'];
                        }
                        
                        if (is_null($item['CDMARCA'])) {
                            $paramsInsMovOpSol['P_CDMARCA'] = "";
                        } else {
                            $paramsInsMovOpSol['P_CDMARCA'] = $item['CDMARCA'];
                        }
                        
                        $this->service->acmInsMovOpSol($paramsInsMovOpSol);
                    }
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function acionaAlteracaoAutomaticaDataEntrega(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $this->service->beginTransaction();
            $this->alteracaoAutomaticaDataEntrega($row);
            $this->service->realizaCommit();
        } catch (Exception $e) {
            $this->service->realizaRollback();
            $response->setError(Error::factoryFromException($e, $e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * ########################################################################
     * ############## >>> ATUALIZAÇÃO AUTOMÁTICA DE PRODUTO <<< ###############
     * ########################################################################
     */
     
    public function calculoAjustes($row) {
        $dbFatoConvOrig;
        $dbFatoConvDest;
        $dbPesoUniOrig;
        $dbPesoUniDest;
        $nedAjustUn;
        $nedAjustQtde;
        
        if ($row["CDPRODUTO_OLD"] != "") {
            $paramsVdProdutoAndAdqAjuste = array(
                "P_CDPRODUTO" => $row["CDPRODUTO_OLD"]
            );
            
            $paramsQryProduto = array(
                "P_CDARVPROD" => $row["CDARVPROD"]
            );
            
            $resultQryVdProduto = $this->service->qryVdProduto($paramsVdProdutoAndAdqAjuste);
            $resultQryProduto = $this->service->qryProduto($paramsQryProduto);
            $resultAdqAjusteQtde = $this->service->adqAjusteQtde($paramsVdProdutoAndAdqAjuste);
            
            if ($resultQryVdProduto["VRFATOCONV"] == "") {
                $dbFatoConvOrig = 1;
            } else {
                $dbFatoConvOrig = $resultQryVdProduto["VRFATOCONV"];
            }
            
            if ($resultQryProduto["VRFATOCONV"] == "") {
                $dbFatoConvDest = 1;
            } else {
                $dbFatoConvDest = $resultQryProduto["VRFATOCONV"];
            }
            
            if ($resultQryVdProduto["VRPESOUNID"] == "") {
                $dbPesoUniOrig = 1;
            } else {
                $dbPesoUniOrig = $resultQryVdProduto["VRPESOUNID"];
            }
            
            if ($resultQryProduto["VRPESOUNID"] == "") {
                $dbPesoUniDest = 1;
            } else {
                $dbPesoUniDest = $resultQryProduto["VRPESOUNID"];
            }
            
            $nedAjustUn = ((($dbFatoConvOrig * $dbPesoUniOrig)/($dbFatoConvDest * $dbPesoUniDest)) - 1) * 100;
            
            if (!empty($resultAdqAjusteQtde)) {
                foreach ($resultAdqAjusteQtde as $result) {
                    if ($result["CDPRODORIG"] == $resultQryVdProduto["CDPRODUTO"] && $result["CDPRODDEST"] == $resultQryProduto["CDPRODUTO"]) {
                        $nedAjustQtde = $result["VRPEAJQTDSOL"];
                    }
                }
            }
            
            if (isset($nedAjustUn)) {
                if($nedAjustUn != 0)
                    $nedAjustUn = preg_replace('/(-?\d+)\.(\d{2})\d*/', '$1, $2', $this->bcconv($nedAjustUn));
            } else {
                $nedAjustUn = NULL;
            }
            
            if (isset($nedAjustQtde)) {
                $nedAjustQtde = preg_replace('/(-?\d+)\.(\d{2})\d*/', '$1,$2', $this->bcconv($nedAjustQtde));
            } else {
                $nedAjustQtde = NULL;
            }
            
            return [ "AJUSTEUN" => $nedAjustUn, "AJUSTEQTDE" => $nedAjustQtde ];
        }
    }
    
    public function acionaCalculoAjustes(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $result = $this->calculoAjustes($row);
            $response->addDataSet(new DataSet('data', [$result]));
        } catch (Exception $e) {
            $response->setError(Error::factoryFromException($e, $e->getMessage(), $e->getCode()));
        }
    }
     
    public function alteracaoAutomaticaProduto($row) {
        $row["P_ALTAUT"] = isset($row["DSJUSTOPER_NEW"]) ? $row["DSJUSTOPER_NEW"] : $this->getMensagemPadrao('Alt_Auto');
        $dbAux       = 0.0;
        $dbQtdeAlter = 0.0;
        $boCommit    = true;
        $stProdOrig  = "";
        $rowsSemSegMerc = array();
        $rowsSemProdFiliAtivo = array();
        $rowsAlteradas = array();
        
        self::parseQuantidadesToFloat($row);
        
        try {
            $this->service->beginTransaction();
            
            foreach ($row["ITENS_SELECIONADOS"] as $item) {
                self::parseQuantidadesToFloat($item);
                self::parseDates($item);
                
                $cdSegMerc = $this->service->getCdSegMercFilial($item["CDFILSOLI"]);
            
                if($cdSegMerc) {
                    $res = $this->service->verificaSegMercado($item);
                    
                    if(empty($res) || count($res) == 0) {
                        $rowsSemSegMerc[] = $item;
                        continue;
                    }
                }
                
                $res1 = $this->service->verificaProdFiliAtivo($item);
                
                if(empty($res1) || count($res1) == 0) {
                    $rowsSemProdFiliAtivo[] = $item;
                    continue;
                }
                
                /**
                 * QTANTOPER equivale a QUANTIDADE no Grid
                 */
                $item["QTANTOPER"] = $item["QTSOLIPROD"];
                
                $dbAux = $item["QTANTOPER"];
                
                if (empty($stProdOrig)) {
                    $stProdOrig = $item["CDPRODUTO"];
                }
                 
                if ($row["P_IDPRODUTO"]) {
                    $paramsAdqArredProduto = array(
                        "P_CDPRODUTO" => $row["P_CDPRODUTO"]
                    );
                      
                    $arredProduto = $this->service->adqArredProduto($paramsAdqArredProduto);
                } else {
                    $paramsAdqArredProduto = array(
                        "P_CDPRODUTO" => $item["CDPRODUTO"]
                    );
                    
                    $arredProduto = $this->service->adqArredProduto($paramsAdqArredProduto);
                }
                
                if ($row["P_IDPRODUTO"]) {
                    $paramsAdqVerifItemSol = array(
                       "P_CDFILSOLI"    => $item["CDFILSOLI"],
                       "P_CDFILLAN"     => $item["CDFILLAN"],
                       "P_NRSOLICMP"    => $item["NRSOLICMP"],
                       "P_CDPRODUTO"    => $row["P_CDPRODUTO"],
                       "P_DTPROGENTR"   => $item["DTPROGENTR"],
                    );
                    $itemSol = $this->service->adqVerifItemSol($paramsAdqVerifItemSol);
                    
                    $paramsAdqProdFili = array(
                       "P_CDFILIAL"     => $item["CDFILSOLI"],
                       "P_CDPRODUTO"    => $row["P_CDPRODUTO"],
                    );
                    $prodFili = $this->service->adqProdFili($paramsAdqProdFili);
                    
                    $paramsAdqProdFili1 = array(
                       "P_CDFILIAL"     => $item["CDFILSOLI"],
                       "P_CDPRODUTO"    => $item["CDPRODUTO"],
                    );
                    
                    $prodFili1 = $this->service->adqProdFili1($paramsAdqProdFili1);
                    
                    if (empty($prodFili) && empty($prodFili1)) {
                      if (empty($itemSol)) {
                            $paramsAcmInsertItemSoli = array(
                                "CDFILSOLI"     => $item["CDFILSOLI"],
                                "CDFILLAN"      => $item["CDFILLAN"],
                                "NRSOLICMP"     => $item["NRSOLICMP"],
                                "CDPRODUTO"     => $item["CDPRODUTO"],
                                "DTPROGENTR"    => $item["DTPROGENTR"],
                                "PRODNOVO"      => $row["P_CDPRODUTO"],
                            );
                            
                            if ($row["P_IDQTDE"]) {
                                //echo(">> Quantidade PREENCHIDO no front <<\n");
                                $dbAux = $row["P_QTDE"];
                            } else {
                                $dbAux = round(($item["QTANTOPER"] + (($row["P_AJUSTEUN"] / 100) * $item["QTANTOPER"])) +
                                         (($row["P_AJUSTEQTDE"] / 100) * ($item["QTANTOPER"] + (($row["P_AJUSTEUN"] / 100) * $item["QTANTOPER"])) ),
                                         3);
                            }
                            
                            if ($arredProduto["VRARREDCMP"] > 0) {
                                if (($dbAux - self::truncate((($dbAux/$arredProduto["VRARREDCMP"]) * $arredProduto["VRARREDCMP"]) != 0))) {
                                    $dbAux = ceil($dbAux/$arredProduto["VRARREDCMP"]) * $arredProduto["VRARREDCMP"];
                                }
                            }
                            
                            $paramsAcmInsertItemSoli["QTSOLIPROD"] = $dbAux;
                            
                            if ($row["P_IDQTDEVOL"]) {
                                $paramsAcmInsertItemSoli["QTVOLUITEMSO"] = $row["P_QTDEVOL"];
                            } else {
                                $paramsAcmInsertItemSoli["QTVOLUITEMSO"] = NULL;
                            }
                            
                            $this->service->acmInsertItemSoli($paramsAcmInsertItemSoli);
                        
                            $paramsAcmInsertMovopSol = array(
                                "P_DTOPERSOL"   => date("d/m/Y"),
                                "P_CDFILSOLMOV" => $item["CDFILSOLI"],
                                "P_CDFILLANMOV" => $item["CDFILLAN"],
                                "P_NRSOLIMOV"   => $item["NRSOLICMP"],
                                "P_CDPRODUTO"   => $item["CDPRODUTO"],
                                "P_DTENTROPER"  => $item["DTPROGENTR"],
                                "P_QTANTOPER"   => $item["QTANTOPER"],
                                "P_QTDEPOPER"   => 0,
                                "P_IDOPERAMOVS" => "E",
                                "P_CDOPERADOR"  => $row["CDOPERADOR"],
                                "P_DSJUSTOPER"  => $row["P_ALTAUT"],
                            );
                            
                            $this->service->acmInsertMovopSol($paramsAcmInsertMovopSol);
                        
                            $paramsAcmInsertMovopSol = array(
                                "P_DTOPERSOL"   => date("d/m/Y"),
                                "P_CDFILSOLMOV" => $item["CDFILSOLI"],
                                "P_CDFILLANMOV" => $item["CDFILLAN"],
                                "P_NRSOLIMOV"   => $item["NRSOLICMP"],
                                "P_CDPRODUTO"   => $row["P_CDPRODUTO"],
                                "P_DTENTROPER"  => $item["DTPROGENTR"],
                                "P_QTANTOPER"   => 0,
                                "P_QTDEPOPER"   => $dbAux,
                                "P_IDOPERAMOVS" => "I",
                                "P_CDOPERADOR"  => $row["CDOPERADOR"],
                                "P_DSJUSTOPER"  => $row["P_ALTAUT"],
                            );
                            
                            $this->service->acmInsertMovopSol($paramsAcmInsertMovopSol);
                        
                            $paramsAdqItemsoliDel = array(
                                "P_CDFILSOLI"   => $item["CDFILSOLI"],
                                "P_CDFILLAN"    => $item["CDFILLAN"],
                                "P_NRSOLICMP"   => $item["NRSOLICMP"],
                                "P_CDPRODUTO"   => $item["CDPRODUTO"],
                                "P_DTPROGENTR"  => $item["DTPROGENTR"],
                            );
                            
                            $itemSoliDel = $this->service->adqItemsoliDel($paramsAdqItemsoliDel);
                            
                            if (empty($itemSoliDel)) {
                                $paramsAcmInsItemSoliDel = array(
                                    "P_CDFILSOLI"   => $item["CDFILSOLI"],
                                    "P_CDFILLAN"    => $item["CDFILLAN"],
                                    "P_NRSOLICMP"   => $item["NRSOLICMP"],
                                    "P_CDPRODUTO"   => $item["CDPRODUTO"],
                                    "P_DTPROGENTR"  => $item["DTPROGENTR"],
                                );
                                   
                                if ($item["QTSOLIORIG"] == 0) {
                                    $paramsAcmInsItemSoliDel["P_QTSOLIORIGDEL"] = $item["QTANTOPER"];
                                } else {
                                    $paramsAcmInsItemSoliDel["P_QTSOLIORIGDEL"] = $item["QTSOLIORIG"];
                                }
                                
                                $this->service->acmInsItemSoliDel($paramsAcmInsItemSoliDel);
                            } else {
                                $paramsAcmUpdItemSoliDel = array(
                                    "P_CDFILSOLI"   => $item["CDFILSOLI"],
                                    "P_CDFILLAN"    => $item["CDFILLAN"],
                                    "P_NRSOLICMP"   => $item["NRSOLICMP"],
                                    "P_CDPRODUTO"   => $item["CDPRODUTO"],
                                    "P_DTPROGENTR"  => $item["DTPROGENTR"]
                                );
                                
                                if ($item["QTSOLIORIG"] == 0) {
                                    $paramsAcmUpdItemSoliDel["P_QTSOLIORIGDEL"] = $item["QTANTOPER"];
                                } else {
                                    $paramsAcmUpdItemSoliDel["P_QTSOLIORIGDEL"] = $item["QTSOLIORIG"];
                                }
                                //throw new Exception('WITHOUT_PERMISSION');
                                //$this->service->acmUpdItemSoliDel($paramsAcmUpdItemSoliDel);
                            }
        
                            $paramsAcmDeleteItemSoli = array(
                                "P_CDFILSOLI"   => $item["CDFILSOLI"],
                                "P_CDFILLAN"    => $item["CDFILLAN"],
                                "P_NRSOLICMP"   => $item["NRSOLICMP"],
                                "P_CDPRODUTO"   => $item["CDPRODUTO"],
                                "P_DTPROGENTR"  => $item["DTPROGENTR"],
                            );
                            
                            $this->service->acmDeleteItemSoli($paramsAcmDeleteItemSoli);
                       } else {
                            $paramsAcmUpdateItemSoli = array(
                                "CDFILSOLI"     => $item["CDFILSOLI"],
                                "CDFILLAN"      => $item["CDFILLAN"],
                                "NRSOLICMP"     => $item["NRSOLICMP"],
                                "CDPRODUTO"     => $row["P_CDPRODUTO"],
                                "DTPROGENTR"    => $item["DTPROGENTR"],
                            );
                            
                            if ($row["P_IDQTDE"]) {
                                $dbAux = $row["P_QTDE"];
                            } else {
                                $dbAux = $itemSol["QTSOLIPROD"] +
                                         ( round(($item["QTANTOPER"] + (($row["P_AJUSTEUN"] / 100) * $item["QTANTOPER"])) +
                                         (($row["P_AJUSTEQTDE"] / 100) * ($item["QTANTOPER"] + (($row["P_AJUSTEUN"] / 100) * $item["QTANTOPER"])) ),
                                         3)
                                          );
                            }
                             
                            if ($arredProduto["VRARREDCMP"] > 0) {
                                if (($dbAux - self::truncate((($dbAux/$arredProduto["VRARREDCMP"]) * $arredProduto["VRARREDCMP"]) != 0))) {
                                    $dbAux = ceil($dbAux/$arredProduto["VRARREDCMP"]) * $arredProduto["VRARREDCMP"];
                                }
                            }
                            
                            $paramsAcmUpdateItemSoli["QTSOLIPROD"] = $dbAux;
                            $paramsAcmUpdateItemSoli["QTSOLIORIG"] = $itemSol["QTSOLIPROD"];
                            
                            if ($row["P_IDQTDEVOL"]) {
                                $paramsAcmUpdateItemSoli["QTVOLUITEMSO"] = $row["P_QTDEVOL"];
                            } else {
                                $paramsAcmUpdateItemSoli["QTVOLUITEMSO"] = NULL;
                            }
                             
                            $this->service->acmUpdateItemSoliAlteracaoAutomatica($paramsAcmUpdateItemSoli);
        
                            $paramsAcmInsertMovopSol = array(
                                "P_DTOPERSOL"   => date("d/m/Y"),
                                "P_CDFILSOLMOV" => $item["CDFILSOLI"],
                                "P_CDFILLANMOV" => $item["CDFILLAN"],
                                "P_NRSOLIMOV"   => $item["NRSOLICMP"],
                                "P_CDPRODUTO"   => $item["CDPRODUTO"],
                                "P_DTENTROPER"  => $item["DTPROGENTR"],
                                "P_QTANTOPER"   => $item["QTANTOPER"],
                                "P_QTDEPOPER"   => 0,
                                "P_IDOPERAMOVS" => "E",
                                "P_CDOPERADOR"  => $row["CDOPERADOR"],
                                "P_DSJUSTOPER"  => $row["P_ALTAUT"],
                            );
                            
                            $this->service->acmInsertMovopSol($paramsAcmInsertMovopSol);
                        
                            $paramsAcmInsertMovopSol = array(
                                "P_DTOPERSOL"   => date("d/m/Y"),
                                "P_CDFILSOLMOV" => $item["CDFILSOLI"],
                                "P_CDFILLANMOV" => $item["CDFILLAN"],
                                "P_NRSOLIMOV"   => $item["NRSOLICMP"],
                                "P_CDPRODUTO"   => $row["P_CDPRODUTO"],
                                "P_DTENTROPER"  => $item["DTPROGENTR"],
                                "P_QTANTOPER"   => $itemSol["QTSOLIPROD"],
                                "P_QTDEPOPER"   => $dbAux,
                                "P_IDOPERAMOVS" => "A",
                                "P_CDOPERADOR"  => $row["CDOPERADOR"],
                                "P_DSJUSTOPER"  => $row["P_ALTAUT"],
                            );
                            
                            $this->service->acmInsertMovopSol($paramsAcmInsertMovopSol);
                        
                            $paramsAcmDeleteItemSoli = array(
                                "P_CDFILSOLI"   => $item["CDFILSOLI"],
                                "P_CDFILLAN"    => $item["CDFILLAN"],
                                "P_NRSOLICMP"   => $item["NRSOLICMP"],
                                "P_CDPRODUTO"   => $item["CDPRODUTO"],
                                "P_DTPROGENTR"  => $item["DTPROGENTR"],
                            );
                            
                            $this->service->acmDeleteItemSoli($paramsAcmDeleteItemSoli);
                       }
                    }
                } else {
                    if ($row["P_IDQTDE"] || $row["P_IDQTDEVOL"]) {
                        $paramsAcmUpdateItemSoli = array(
                            "CDFILSOLI"     => $item["CDFILSOLI"],
                            "CDFILLAN"      => $item["CDFILLAN"],
                            "NRSOLICMP"     => $item["NRSOLICMP"],
                            "CDPRODUTO"     => $item["CDPRODUTO"],
                            "DTPROGENTR"    => $item["DTPROGENTR"],
                        );
                        
                        if ($row["P_IDQTDE"]) {
                            $dbAux = $row["P_QTDE"];
                            if ($arredProduto["VRARREDCMP"] > 0) {
                                if (($dbAux - self::truncate($dbAux/$arredProduto["VRARREDCMP"]) * $arredProduto["VRARREDCMP"]) != 0) {
                                    $dbAux = ceil($dbAux/$arredProduto["VRARREDCMP"]) * $arredProduto["VRARREDCMP"];
                                }
                             }
                             
                             $paramsAcmUpdateItemSoli["QTSOLIPROD"] = $dbAux;
                             
                             if (isset($row["QTSOLIORIG"]) && $row["QTSOLIORIG"] == 0) {
                                $paramsAcmUpdateItemSoli["QTSOLIORIG"] = $item["QTANTOPER"];
                             } else {
                                $paramsAcmUpdateItemSoli["QTSOLIORIG"] = $item["QTSOLIORIG"];
                             }
                        }
                        
                        if ($row["P_IDQTDEVOL"]) {
                            $paramsAcmUpdateItemSoli["QTVOLUITEMSO"] = $row["P_QTDEVOL"];
                        } else {
                            $paramsAcmUpdateItemSoli["QTVOLUITEMSO"] = NULL;
                        }
                        
                        $this->service->acmUpdateItemSoliAlteracaoAutomatica($paramsAcmUpdateItemSoli);
                        
                        if ($row["P_QTDE"] != $item["QTANTOPER"]) {
                            $paramsAcmInsertMovopSol = array(
                                "P_DTOPERSOL"   => date("d/m/Y"),
                                "P_CDFILSOLMOV" => $item["CDFILSOLI"],
                                "P_CDFILLANMOV" => $item["CDFILLAN"],
                                "P_NRSOLIMOV"   => $item["NRSOLICMP"],
                                "P_CDPRODUTO"   => $item["CDPRODUTO"],
                                "P_DTENTROPER"  => $item["DTPROGENTR"],
                                "P_QTANTOPER"   => $item["QTANTOPER"],
                                "P_QTDEPOPER"   => $dbAux,
                                "P_IDOPERAMOVS" => "A",
                                "P_CDOPERADOR"  => $row["CDOPERADOR"],
                                "P_DSJUSTOPER"  => $row["P_ALTAUT"],
                            );
                            
                            $this->service->acmInsertMovopSol($paramsAcmInsertMovopSol);
                        }
                    }
                }
                
                if (!empty($row["P_CDPRODUTO"])) {
                    $paramsAdqProdesto = array(
                        "P_OLDPRODUTO" => $row["CDPRODUTO"],
                        "P_NEWPRODUTO" => $row["P_CDPRODUTO"],
                    );
                    $prodesto = $this->service->adqProdesto($paramsAdqProdesto);
                }
                
                if (!empty($prodesto) || empty($row["P_CDPRODUTO"])) {
                    if (!empty($prodesto)) {
                        $dbAux = (($dbAux * $prodesto["VRFATOCONVNEW"]) / $prodesto["VRFATOCONVOLD"]);
                    }
                    
                    $paramsAdqAutGrSoli = array(
                        "P_CDOPERADOR"  => $row["CDOPERADOR"],
                        "P_CDFILIAL"    => $row["P_FILIPROC"]
                    );
                    $autGrSoli = $this->service->adqAutGrSoli($paramsAdqAutGrSoli);
                    
                    /**
                     * No Delphi, cds e dsp são 'datasources' da query adqAutGrSoli.
                     */
                    $cdsAutGrSoli = $dspAutGrSoli = $autGrSoli;
                    
                    // if ($item["IDTIPLAN"] == 'A') {
                    //     /*if ($item["QTSOLIORIG"] == 0) {
                    //         $dbQtdeAlter = $item["QTANTOPER"];
                    //     } else {
                    //         $dbQtdeAlter = $item["QTSOLIORIG"];
                    //     }*/
                    //     $dbQtdeAlter = $item["QTANTOPER"];
                        
                    //     if ($row["P_IDQTDE"]) {
                    //         if ($row["P_IDBLOQALTSOLAUT"] == 'S') {
                    //             if ($boCommit) {
                    //                 if ($dbAux > $item["QTANTOPER"]) {
                    //                     $aumentaQtde = self::searchInArray($autGrSoli, "IDAUTALTEPRODSOLI", "AUMENTAR_QUANTIDADE");
                                        
                    //                     if (!$aumentaQtde) {
                    //                         $boCommit = false;
                    //                     } else {
                    //                         if (!is_null($aumentaQtde["PEALTEQTDESOLI"])) {
                    //                             if ($dbAux > ($dbQtdeAlter + ($dbQtdeAlter * ($aumentaQtde["PEALTEQTDESOLI"]/100)))) {
                    //                                 $boCommit = false;
                    //                             }
                    //                         }
                    //                     }
                    //                 }
                    //             }
                                
                    //             if ($boCommit) {
                    //                 if ($dbAux < $item["QTANTOPER"]) {
                    //                     $diminuiQtde = self::searchInArray($autGrSoli, "IDAUTALTEPRODSOLI", "DIMINUIR_QUANTIDADE");
                                        
                    //                     if (!$diminuiQtde) {
                    //                         $boCommit = false;
                    //                     } else {
                    //                         if (!is_null($diminuiQtde["PEALTEQTDESOLI"])) {
                    //                             if ($dbAux < ($dbQtdeAlter - ($dbQtdeAlter * ($diminuiQtde["PEALTEQTDESOLI"]/100)))) {
                    //                                 $boCommit = false;
                    //                             }
                    //                         }
                    //                     }
                    //                 }
                    //             }
                    //         }
                    //     }
                    // }
                }
                
            $rowsAlteradas[] = $item;
            }
            
            if ($row["P_AJUSTEQTDE"] > 0) {
                $paramsAdqExisteAjuste = array(
                    "P_CDPRODORIG" => $stProdOrig,
                    "P_CDPRODDEST" => $row["P_CDPRODUTO"],
                );
                
                $ajuste = $this->service->adqExisteAjuste($paramsAdqExisteAjuste);
                
                if (empty($ajuste)) {
                    $paramsAcmInsAjuste = array(
                       "P_AJUSTEQTDE" => $row["P_AJUSTEQTDE"],
                       "P_CDPRODDEST" => $row["P_CDPRODUTO"],
                       "P_CDPRODORIG" => $stProdOrig,
                    );
                    
                    $this->service->acmInsAjuste($paramsAcmInsAjuste);
                } else {
                    if ($ajuste["VRPEAJQTDSOL"] != $row["P_AJUSTEQTDE"]) {
                        $paramsAcmUpdAjuste = array(
                          "P_AJUSTEQTDE" => $row["P_AJUSTEQTDE"],
                          "P_CDPRODDEST" => $row["P_CDPRODUTO"],
                          "P_CDPRODORIG" => $stProdOrig
                        );
                        
                        $this->service->acmUpdAjuste($paramsAcmUpdAjuste);
                    }
                }
            }
            
            if ($boCommit == true) {
                $this->service->realizaCommit();
                
                $ret = array(
                    "RAT" => $rowsAlteradas,
                    "RSM" => $rowsSemSegMerc,
                    "RSP" => $rowsSemProdFiliAtivo
                );
                
                return $ret;
            } else {
                $this->service->realizaRollback();
            }
        } catch (Exception $e) {
            $this->service->realizaRollback();
            throw $e;
        }
    }
    
    public function acionaAlteracaoAutomaticaProduto(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $this->service->beginTransaction();
            $resp = $this->alteracaoAutomaticaProduto($row);
            
            if(count($resp["RAT"]) == 0) {
                $response->addMessage(new Message("NO_ROWS_UPDATED"));
            } else {
                if(count($resp["RSM"]) == 0 && count($resp["RSP"]) == 0) {
                    $response->addMessage(new Message("ALL_ROWS_UPDATED"));
                } else if(count($resp["RSM"]) > 0 || count($resp["RSP"]) || 0) {
                    $response->addMessage(new Message("SOME_ROWS_UPDATED"));
                }
            }
            
            $this->service->realizaCommit();
        } catch (Exception $e) {
            $this->service->realizaRollback();
            $response->setError(Error::factoryFromException($e, $e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * ########################################################################
     * ########################## >>> UTILIDADE <<< ###########################
     * ########################################################################
     */
    
    public function getMensagemPadrao($tipoMensagem) {
        return $this->mensagensPadrao[$tipoMensagem];
    }
    
    /**
     * Converte um valor float recebido do frontend no padrão brasileiro
     * para o valor de float no padrão americano.
     * 
     * * Padrão brasileiro - vírgula na separação da casa decimal;
     * * Padrão americano  - ponto na separação da casa decimal.
     * 
     * @param   string $floatVar String contendo o valor em float no padrão brasileiro.
     * @return  float Valor convertido no padrão americano.
     */
    public function parseFloat($floatVar){
        if (is_string($floatVar)) {
            $floatVar = str_replace(".", "", $floatVar);
            $floatVar = str_replace(",", ".", $floatVar);
        }
        
        return (float) $floatVar;
    }
    
    /**
     * Converte todas as quantidades do item para float utilizando o parseFloat.
     * 
     * Obs.: As quantidades devem estar listadas no campo interno da função.
     * 
     * @param   array $item Item contendo as quantidades a serem convertidas.
     */
    public function parseQuantidadesToFloat(&$item){
        $fields = ["QTSOLIPROD", "QTSOLIPROD_OLD", "QTSOLIORIG", "QTVOLUITEMSO", "P_QTDE", "P_QTDEVOL", "P_AJUSTEUN", "P_AJUSTEQTDE"];
        
        foreach ($fields as $field) {
            if (array_key_exists($field, $item)) {
                $item[$field] = self::parseFloat($item[$field]);
            }
        }
    }
    
    /**
     * Formata todas as datas do item no padrão BR dd/mm/YYYY.
     * 
     * Obs.: As datas devem estar listadas no campo interno da função.
     * 
     * @param   array $item Item contendo as datas a serem convertidas.
     */
    public function parseDates(&$item) {
        $fields = ["DTPROGENTR", "DTPROGENTR_OLD", "DTPROGENTR_NEW", "DTUTILIZA"];
        foreach ($fields as $field) {
            if (isset($item[$field])) {
                $item[$field] = DateUtil::truncateString($item[$field]);
            }
        }
    }
    
    /**
     * Trunca o valor recebido.
     * 
     * @param   string/float $valor Valor a ser truncado.
     * @return  float Valor truncado.
     */
    public function truncate($valor){
        return (float) explode(".", $valor)[0];
    }
    
    /**
    * Convert a number to locale independent string without E notation and without
    * loosing precision
    *
    * @param      int/float/double $fNumber The number to convert.
    * @return     string The locale independent converted number.
    * @author     http://php.net/manual/pt_BR/ref.bc.php#92888
    */
    public function bcconv($fNumber) {
        $sAppend = '';
        (integer)$iDecimals = ini_get('precision') - floor(log10(abs($fNumber)));
        if (0 > $iDecimals) {
            $fNumber *= pow(10, $iDecimals);
            $sAppend = str_repeat('0', -$iDecimals);
            $iDecimals = 0;
        }
        return number_format($fNumber, $iDecimals, '.', '').$sAppend;
    }
    
    private function searchInArray($array, $property, $valueToSearch) {
	    foreach ($array as $item) {
	        if (isset($item[$property]) && ($item[$property] == $valueToSearch)) {
	            return $item;
	        }
	    }
	    
	    return NULL;
	}
	
	public function getPermissoesAlt(DTO\Request\Row $request, DTO\Response $response) {
        $permissoes = $this->service->queryAutGrSoli();
        
        $response->addDataSet(new DataSet('permissoes', $permissoes)); 
    }
}