<?php
namespace Teknisa\CMP\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;

class AtualizacaoSolicitacaoService
{
    protected $connection;
    protected $environment;
    protected $request;
    protected $response;
    
    public function __construct($connection, $environment) 
    {
        $this->connection = $connection;
        $this->environment = $environment;
    }
    
    public function getNrOrg()
    {
        return $this->environment->getNrOrg();
    }
    
    public function executaSql($sql, $params, $consulta = true) {
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
            if($consulta == true) {
                return $std->fetchAll();
            }
            
            return true;
        } catch (\Exception $e){
            throw $e;
        } 
    }
    
    public function updateItemSoliRelacAF($row)
    {
        $update = "UPDATE ITEMSOLIRELACAF
                      SET DTPROGENTAF = :P_DTPROGENTNEW
                    WHERE ITEMSOLIRELACAF.CDFILDEST    = :P_CDFILDEST
                      AND ITEMSOLIRELACAF.CDFILLANC    = :P_CDFILLANC
                      AND ITEMSOLIRELACAF.NRAUTOFORN   = :P_NRAUTOFORN
                      AND ITEMSOLIRELACAF.CDMARCAAF    = :P_CDMARCA
                      AND ITEMSOLIRELACAF.CDPRODUTOAF  = :P_CDPRODUTO
                      AND ITEMSOLIRELACAF.DTPROGENTAF  = :P_DTPROGENTOLD
                      AND ITEMSOLIRELACAF.CDCENTITMAP  = :P_CDFILCENTCMP
                      AND ITEMSOLIRELACAF.NRSEQITMAP   = :P_NRSEQAPURAF
                      AND ITEMSOLIRELACAF.NRORG        = :NRORG
                      AND NOT EXISTS(SELECT 1
                                       FROM ITEMSOLIRELACAF R
                                      WHERE R.CDFILSOLI    = ITEMSOLIRELACAF.CDFILSOLI
                                        AND R.CDFILLAN     = ITEMSOLIRELACAF.CDFILLAN
                                        AND R.NRSOLICMP    = ITEMSOLIRELACAF.NRSOLICMP
                                        AND R.CDPRODUTO    = ITEMSOLIRELACAF.CDPRODUTO
                                        AND R.DTPROGENTR   = ITEMSOLIRELACAF.DTPROGENTR
                                        AND R.CDCENTITMAP  = :P_CDFILCENTCMP
                                        AND R.NRSEQITMAP   = :P_NRSEQAPURAF
                                        AND R.CDFILDEST    = :P_CDFILDEST
                                        AND R.CDFILLANC    = :P_CDFILLANC
                                        AND R.NRAUTOFORN   = :P_NRAUTOFORN
                                        AND R.CDMARCAAF    = :P_CDMARCA
                                        AND R.CDPRODUTOAF  = :P_CDPRODUTO
                                        AND R.DTPROGENTAF  = :P_DTPROGENTNEW
                                        AND R.NRORG        = :NRORG)";

        $params = [
            'P_CDFILDEST'    => $row['CDFILDEST'],
            'P_CDFILLANC'    => $row['CDFILLANC'],
            'P_NRAUTOFORN'   => $row['NRAUTOFORN'],
            'P_CDMARCA'      => $row['CDMARCA'],
            'P_CDPRODUTO'    => $row['CDPRODUTO'],
            
            'P_CDFILCENTCMP' => $row['CDFILCENTCMP'],
            'P_NRSEQAPURAF'  => $row['NRAPURPREC'],
            'P_DTPROGENTNEW' => $row['DTPROGENTAF'],
            'NRORG' => $this->environment->getNrOrg()
        ];
        
        return self::executaSql($update, $params, false);
    }
    
    public function updateQtSoliProd($itemsoli, $quantDife)
    {
        $update = "UPDATE ITEMSOLI 
                      SET QTSOLIPROD = QTSOLIPROD + :P_QTSOLIPROD
                    WHERE CDFILSOLI  = :P_CDFILSOLI
                      AND CDFILLAN   = :P_CDFILLAN
                      AND NRSOLICMP  = :P_NRSOLICMP
                      AND CDPRODUTO  = :P_CDPRODUTO
                      AND DTPROGENTR = :P_DTPROGENTR
                      AND NRORG      = :NRORG";

        $params = [
            'P_QTSOLIPROD' => $quantDife,
            'P_CDFILSOLI'  => $itemsoli['CDFILSOLI'],
            'P_CDFILLAN'   => $itemsoli['CDFILLAN'],
            'P_NRSOLICMP'  => $itemsoli['NRSOLICMP'],
            'P_CDPRODUTO'  => $itemsoli['CDPRODUTO'],
            'P_DTPROGENTR' => $itemsoli['DTPROGENTR'],
            'NRORG' => $this->environment->getNrOrg()
        ];

        return self::executaSql($update, $params, false);
    }
    
    public function updateQtSoliProdApu($row, $itemsoli, $quantDife)
    {
        $update = "UPDATE ITEMSOLIRELACAF
                      SET QTSOLIPROD = QTSOLIPROD + :P_QTSOLIPROD
                    WHERE CDFILSOLI   = :P_CDFILSOLI
                      AND CDFILLAN    = :P_CDFILLAN      
                      AND NRSOLICMP   = :P_NRSOLICMP
                      AND CDPRODUTO   = :P_CDPRODUTO
                      AND DTPROGENTR  = :P_DTPROGENTR
                      AND CDCENTITMAP = :P_CDFILCENTCMP
                      AND NRSEQITMAP  = :P_NRSEQAPURAF
                      AND CDFILDEST   = :P_CDFILDEST
                      AND CDFILLANC   = :P_CDFILLANC
                      AND NRAUTOFORN  = :P_NRAUTOFORN
                      AND CDMARCAAF   = :P_CDMARCA
                      AND CDPRODUTOAF = :P_CDPRODUTOAF
                      AND DTPROGENTAF = :P_DTPROGENTAF
                      AND NRORG       = :NRORG";
                      
        $params = [
            'P_QTSOLIPROD'   => $quantDife,
            'P_CDFILSOLI'    => $itemsoli['CDFILSOLI'],
            'P_CDFILLAN'     => $itemsoli['CDFILLAN'],
            'P_NRSOLICMP'    => $itemsoli['NRSOLICMP'],
            'P_CDPRODUTO'    => $itemsoli['CDPRODUTO'],
            'P_DTPROGENTR'   => $itemsoli['DTPROGENTR'],
            
            'P_NRSEQAPURAF'  => $row['NRSEQAPURAF'],
            'P_CDFILCENTCMP' => $row['CDFILCENTCMP'],
            'P_CDFILDEST'    => $row['CDFILDEST'],
            'P_CDFILLANC'    => $row['CDFILLANC'],
            'P_NRAUTOFORN'   => $row['NRAUTOFORN'],
            'P_CDPRODUTOAF'  => $row['CDPRODUTO'],
            'P_CDMARCA'      => $row['CDMARCA'],
            'P_DTPROGENTAF'  => $row['DTPROGENTAF'],
            
            'NRORG' => $this->environment->getNrOrg()
        ];

        return self::executaSql($update, $params, false);
    }
    
    public function insertMovOpSol($row, $itemsoli, $quantDife, $idOperation)
    {
        $insert = "INSERT INTO MOVOPSOL
                    (CDFILSOLMOV, CDFILLANMOV, NRSOLIMOV, CDPRODUTO, DTENTROPER,
                     DTOPERSOL, CDOPERADOR, QTANTOPER, QTDEPOPER, DSJUSTOPER,
                     IDOPERAMOVS, DSRDMUSUMOVS, DSENDUSUMOVS, DSNMUSUMOVS, 
                     DSMETUSUMOVS, NRORG)
                   VALUES
                    (:P_CDFILSOLMOV, :P_CDFILLANMOV, :P_NRSOLIMOV, :P_CDPRODUTO, 
                     :P_DTENTROPER, SYSDATE, :P_CDOPERADOR, :P_QTANTOPER, 
                     :P_QTDEPOPER, :P_DSJUSTOPER, :P_IDOPERAMOVS, 'ANALISE_AF', 'ZEEDHI', 
                     'ZEEDHI', 'ZEEDHI', :NRORG)";
                     
        $params = [
            'P_CDFILSOLMOV' => $itemsoli['CDFILSOLI'],
            'P_CDFILLANMOV' => $itemsoli['CDFILLAN'],
            'P_NRSOLIMOV'   => $itemsoli['NRSOLICMP'],
            'P_CDPRODUTO'   => $itemsoli['CDPRODUTO'],
            'P_DTENTROPER'  => $itemsoli['DTPROGENTR'],
            'P_QTANTOPER'   => $itemsoli['QTSOLIPROD'],
            'P_QTDEPOPER'   => $itemsoli['QTSOLIPROD'] + $quantDife,
            'P_DSJUSTOPER'  => $row['DSJUSTOPERMOV'],
            'P_IDOPERAMOVS' => $idOperation,
            'P_CDOPERADOR'  => $this->environment->getCdOperador(),
            'NRORG'         => $this->environment->getNrOrg()
        ];

        return self::executaSql($insert, $params, false);
    }
    
    public function deleteItemSoli($row)
    {
        $delete = "DELETE FROM ITEMSOLI
                    WHERE CDFILSOLI  = :P_CDFILSOLI
                      AND CDFILLAN   = :P_CDFILLAN
                      AND NRSOLICMP  = :P_NRSOLICMP
                      AND CDPRODUTO  = :P_CDPRODUTO
                      AND DTPROGENTR = :P_DTPROGENTR
                      AND NRORG      = :NRORG";

        $params = [
            'P_CDFILSOLI'  => $row['CDFILSOLI'],
            'P_CDFILLAN'   => $row['CDFILLAN'],
            'P_NRSOLICMP'  => $row['NRSOLICMP'],
            'P_CDPRODUTO'  => $row['CDPRODUTO'],
            'P_DTPROGENTR' => $row['DTPROGENTR'],
            'NRORG'        => $this->environment->getNrOrg()
        ];

        return self::executaSql($delete, $params, false);
    }
}
